@@ -121,11 +121,13 @@ class XMLHandler:
         if not discard_ids and dom_iter.hasAttribute('unique_id'):
             unique_id = long(dom_iter.attributes['unique_id'].value)
         else: unique_id = self.dad.node_id_get()
-        if dom_iter.hasAttribute('tags'): node_tags = dom_iter.attributes['tags'].value
-        else: node_tags = ""
+        node_tags = dom_iter.attributes['tags'].value if dom_iter.hasAttribute('tags') else ""
         if node_tags: self.dad.tags_add_from_node(node_tags)
-        if dom_iter.hasAttribute('readonly'): readonly = (dom_iter.attributes['readonly'].value == "True")
-        else: readonly = False
+        readonly = (dom_iter.attributes['readonly'].value == "True") if dom_iter.hasAttribute('readonly') else False
+        custom_icon_id = int(dom_iter.attributes['custom_icon_id'].value) if dom_iter.hasAttribute('custom_icon_id') else 0
+        is_bold = (dom_iter.attributes['is_bold'].value == "True") if dom_iter.hasAttribute('is_bold') else False
+        foreground = dom_iter.attributes['foreground'].value if dom_iter.hasAttribute('foreground') else ""
+        if not foreground: foreground = None
         syntax_highlighting = dom_iter.attributes['prog_lang'].value
         if syntax_highlighting not in [cons.RICH_TEXT_ID, cons.PLAIN_TEXT_ID]\
         and syntax_highlighting not in self.dad.available_languages:
@@ -133,7 +135,7 @@ class XMLHandler:
             if syntax_highlighting not in self.dad.available_languages:
                 syntax_highlighting = cons.RICH_TEXT_ID
         node_depth = 0 if not tree_father else self.dad.treestore.iter_depth(tree_father)+1
-        cherry = self.dad.get_node_icon(node_depth, syntax_highlighting)
+        cherry = self.dad.get_node_icon(node_depth, syntax_highlighting, custom_icon_id)
         curr_buffer = self.dad.buffer_create(syntax_highlighting)
         if syntax_highlighting != cons.RICH_TEXT_ID: curr_buffer.begin_not_undoable_action()
         # loop into rich text, write into the buffer
@@ -159,7 +161,10 @@ class XMLHandler:
                                                             node_sequence,
                                                             node_tags,
                                                             readonly,
-                                                            None])
+                                                            None,
+                                                            custom_icon_id,
+                                                            support.get_pango_weight(is_bold),
+                                                            foreground])
         self.dad.nodes_names_dict[unique_id] = self.dad.treestore[tree_iter][1]
         if discard_ids:
             # we are importing nodes
@@ -313,6 +318,11 @@ class XMLHandler:
         dom_iter.setAttribute("prog_lang", programming_language)
         dom_iter.setAttribute("tags", self.dad.treestore[tree_iter][6])
         dom_iter.setAttribute("readonly", str(self.dad.treestore[tree_iter][7]))
+        dom_iter.setAttribute("custom_icon_id", str(self.dad.treestore[tree_iter][9]))
+        dom_iter.setAttribute("is_bold", str(support.get_pango_is_bold(self.dad.treestore[tree_iter][10])))
+        foreground = self.dad.treestore[tree_iter][11]
+        if not foreground: foreground = ""
+        dom_iter.setAttribute("foreground", foreground)
         dom_father.appendChild(dom_iter)
         # allocate and init the rich text attributes
         self.curr_attributes = {}
@@ -518,7 +528,7 @@ class XMLHandler:
                     curr_node_level = self.dad.treestore.iter_depth(node_tree_iter)
                     node_name = self.dad.treestore[node_tree_iter][1].replace(cons.CHAR_NEWLINE, cons.CHAR_SPACE).replace(cons.CHAR_CR, "")
                     tag_names = [self.dad.apply_tag_exist_or_create(tag_property, property_value)]
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 2*curr_node_level*cons.CHAR_SPACE + cons.CHAR_LISTARR + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 2*curr_node_level*cons.CHAR_SPACE + self.dad.chars_toc[0] + cons.CHAR_SPACE)
                     curr_offset += 2 + 2*curr_node_level
                     text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_NEWLINE)
                     text_buffer.insert_with_tags_by_name(text_buffer.get_iter_at_offset(curr_offset), node_name, *tag_names)
@@ -529,13 +539,13 @@ class XMLHandler:
                 tag_names = [self.dad.apply_tag_exist_or_create(tag_property, property_value + cons.CHAR_SPACE + element[0])]
                 if not element[0]: continue
                 if element[0][:2] == cons.TAG_PROP_H1:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (2+2*curr_node_level)*cons.CHAR_SPACE + cons.CHARS_LISTBUL[0] + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (2+2*curr_node_level)*cons.CHAR_SPACE + self.dad.chars_toc[1] + cons.CHAR_SPACE)
                     curr_offset += 4 + 2*curr_node_level
                 elif element[0][:2] == cons.TAG_PROP_H2:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (4+2*curr_node_level)*cons.CHAR_SPACE + cons.CHARS_LISTBUL[0] + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (4+2*curr_node_level)*cons.CHAR_SPACE + self.dad.chars_toc[2] + cons.CHAR_SPACE)
                     curr_offset += 6 + 2*curr_node_level
                 else:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (6+2*curr_node_level)*cons.CHAR_SPACE + cons.CHARS_LISTBUL[0] + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (6+2*curr_node_level)*cons.CHAR_SPACE + self.dad.chars_toc[3] + cons.CHAR_SPACE)
                     curr_offset += 8 + 2*curr_node_level
                 text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_NEWLINE)
                 text_buffer.insert_with_tags_by_name(text_buffer.get_iter_at_offset(curr_offset), element[1], *tag_names)
@@ -545,7 +555,7 @@ class XMLHandler:
                 curr_offset += 1
             text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_NEWLINE)
         return toc_list_per_node
-        
+
     def toc_insert_all_iter(self, top_tree_iter):
         """Iterate on nodes for toc_insert_all"""
         self.dad.treeview_safe_set_cursor(top_tree_iter)
@@ -618,13 +628,13 @@ class XMLHandler:
                 tag_names = []
                 tag_names.append(self.dad.apply_tag_exist_or_create(tag_property, property_value + cons.CHAR_SPACE + element[0]))
                 if element[0][:2] == cons.TAG_PROP_H1:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHARS_LISTBUL[0] + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), self.dad.chars_toc[1] + cons.CHAR_SPACE)
                     curr_offset += 2
                 elif element[0][:2] == cons.TAG_PROP_H2:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 2*cons.CHAR_SPACE + cons.CHARS_LISTBUL[0] + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 2*cons.CHAR_SPACE + self.dad.chars_toc[2] + cons.CHAR_SPACE)
                     curr_offset += 4
                 else:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 4*cons.CHAR_SPACE + cons.CHARS_LISTBUL[0] + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 4*cons.CHAR_SPACE + self.dad.chars_toc[3] + cons.CHAR_SPACE)
                     curr_offset += 6
                 text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_NEWLINE)
                 text_buffer.insert_with_tags_by_name(text_buffer.get_iter_at_offset(curr_offset), element[1], *tag_names)
@@ -796,7 +806,7 @@ class StateMachine:
         self.visited_nodes_list = []
         self.visited_nodes_idx = 0
 
-    def requested_previous_visited(self):
+    def requested_visited_previous(self):
         """Requested the Previous Visited Node"""
         if self.visited_nodes_idx != None and self.visited_nodes_idx > 0:
             self.visited_nodes_idx -= 1
@@ -805,7 +815,7 @@ class StateMachine:
             #print "self.visited_nodes_idx", self.visited_nodes_idx
             return None
 
-    def requested_next_visited(self):
+    def requested_visited_next(self):
         """Requested the Next Visited Node"""
         if self.visited_nodes_idx != None and self.visited_nodes_idx < len(self.visited_nodes_list) - 1:
             self.visited_nodes_idx += 1
@@ -817,9 +827,11 @@ class StateMachine:
 
     def node_selected_changed(self, node_id):
         """When a New Node is Selected"""
-        if not self.dad.go_bk_fw_click:
+        if self.dad.user_active and not self.dad.go_bk_fw_click:
             last_index = len(self.visited_nodes_list) - 1
             if self.visited_nodes_idx != last_index: del self.visited_nodes_list[self.visited_nodes_idx+1:last_index+1]
+            if node_id in self.visited_nodes_list:
+                self.visited_nodes_list.remove(node_id)
             self.visited_nodes_list.append(node_id)
             self.visited_nodes_idx = len(self.visited_nodes_list) - 1
         if node_id not in self.nodes_vectors:
@@ -842,7 +854,7 @@ class StateMachine:
         elif alphanum == None: # self.nodes_indicators[node_id] == 2 and non alphanumeric transition
             self.update_state()
 
-    def requested_previous_state(self, node_id):
+    def requested_state_previous(self, node_id):
         """A Previous State, if Existing, is Requested"""
         if self.curr_index_is_last_index(node_id):
             self.update_state()
@@ -850,11 +862,11 @@ class StateMachine:
             self.nodes_indexes[node_id] -= 1
         return self.nodes_vectors[node_id][self.nodes_indexes[node_id]]
 
-    def requested_current_state(self, node_id):
+    def requested_state_current(self, node_id):
         """The current state is requested"""
         return self.nodes_vectors[node_id][self.nodes_indexes[node_id]]
 
-    def requested_subsequent_state(self, node_id):
+    def requested_state_subsequent(self, node_id):
         """A Subsequent State, if Existing, is Requested"""
         if self.nodes_indexes[node_id] < (len(self.nodes_vectors[node_id])-1):
             self.nodes_indexes[node_id] += 1
