@@ -25,17 +25,24 @@ import cons, config
 
 def auto_decode_str(in_str, from_clipboard=False):
     """Try to Detect Encoding and Decode"""
+    try:
+        import chardet
+        enc_dict = chardet.detect(in_str)
+        encoding = enc_dict["encoding"]
+        encodings = [encoding]
+    except:
+        encodings = []
     if in_str.startswith("\xEF\xBB\xBF"): # UTF-8 "BOM"
-        encodings = ["utf-8-sig"]
+        encodings += ["utf-8-sig"]
     elif in_str.startswith(("\xFF\xFE", "\xFE\xFF")): # UTF-16 BOMs
-        encodings = [cons.STR_UTF16]
+        encodings += [cons.STR_UTF16]
     elif from_clipboard:
         if "html" in in_str or "HTML" in in_str:
-            encodings = [cons.STR_UTF8]
+            encodings = [cons.STR_UTF8] + encodings
         else:
-            encodings = [cons.STR_UTF16, cons.STR_UTF8]
+            encodings = [cons.STR_UTF16, cons.STR_UTF8] + encodings
     else:
-        encodings = [cons.STR_UTF16, "utf-16le", cons.STR_UTF8, cons.STR_ISO_8859, locale.getdefaultlocale()[1]]
+        encodings += [cons.STR_UTF16, "utf-16le", cons.STR_UTF8, cons.STR_ISO_8859, locale.getdefaultlocale()[1]]
     for enc in encodings:
         try:
             out_str = in_str.decode(enc)
@@ -138,9 +145,9 @@ def on_sourceview_list_change_level(dad, iter_insert, list_info, text_buffer, le
                 bull_idx = prev_list_info["num"]*(-1) - 1
             else:
                 idx_old = list_info["num"]*(-1) - 1
-                idx_offset = idx_old - curr_level % cons.NUM_CHARS_LISTBUL
-                bull_idx = (next_level + idx_offset) % cons.NUM_CHARS_LISTBUL
-            dad.replace_text_at_offset(cons.CHARS_LISTBUL[bull_idx],
+                idx_offset = idx_old - curr_level % len(dad.chars_listbul)
+                bull_idx = (next_level + idx_offset) % len(dad.chars_listbul)
+            dad.replace_text_at_offset(dad.chars_listbul[bull_idx],
                 bull_offset, bull_offset+1, text_buffer)
         else:
             idx = list_info["aux"]
@@ -196,7 +203,7 @@ def on_sourceview_event_after_key_press(dad, text_view, event, syntax_highl):
     if not dad.ctrl_down:
         if keyname in cons.STR_KEYS_CONTROL:
             dad.ctrl_down = True
-    if keyname in [cons.STR_KEY_DQUOTE, cons.STR_KEY_SQUOTE]:
+    if dad.auto_smart_quotes and keyname in [cons.STR_KEY_DQUOTE, cons.STR_KEY_SQUOTE]:
         if syntax_highl in [cons.RICH_TEXT_ID, cons.PLAIN_TEXT_ID]:
             iter_insert = text_buffer.get_iter_at_mark(text_buffer.get_insert())
             if iter_insert:
@@ -272,7 +279,7 @@ def on_sourceview_event_after_key_press(dad, text_view, event, syntax_highl):
             pre_spaces = 3*curr_level*cons.CHAR_SPACE if curr_level else ""
             if list_info["num"] < 0:
                 index = list_info["num"]*(-1) - 1
-                text_buffer.insert(iter_insert, pre_spaces+cons.CHARS_LISTBUL[index]+cons.CHAR_SPACE)
+                text_buffer.insert(iter_insert, pre_spaces+dad.chars_listbul[index]+cons.CHAR_SPACE)
             elif list_info["num"] == 0:
                 text_buffer.insert(iter_insert, pre_spaces+cons.CHAR_LISTTODO+cons.CHAR_SPACE)
             else:
@@ -302,13 +309,13 @@ def on_sourceview_event_after_key_press(dad, text_view, event, syntax_highl):
                         # at line start
                         if iter_start.get_char() == cons.CHAR_LESSER:
                             # "<> " becoming "◇ "
-                            dad.special_char_replace(cons.CHARS_LISTBUL[1], iter_start, iter_insert, text_buffer)
+                            dad.special_char_replace(config.CHARS_LISTBUL_DEFAULT[1], iter_start, iter_insert, text_buffer)
                         elif iter_start.get_char() == cons.CHAR_MINUS:
                             # "-> " becoming "→ "
-                            dad.special_char_replace(cons.CHARS_LISTBUL[4], iter_start, iter_insert, text_buffer)
+                            dad.special_char_replace(config.CHARS_LISTBUL_DEFAULT[4], iter_start, iter_insert, text_buffer)
                         elif iter_start.get_char() == cons.CHAR_EQUAL:
                             # "=> " becoming "⇒ "
-                            dad.special_char_replace(cons.CHARS_LISTBUL[5], iter_start, iter_insert, text_buffer)
+                            dad.special_char_replace(config.CHARS_LISTBUL_DEFAULT[5], iter_start, iter_insert, text_buffer)
                     elif iter_start.get_char() == cons.CHAR_MINUS and iter_start.backward_char():
                         if iter_start.get_char() == cons.CHAR_LESSER:
                             # "<-> " becoming "↔ "
@@ -349,7 +356,7 @@ def on_sourceview_event_after_key_press(dad, text_view, event, syntax_highl):
                         dad.special_char_replace(cons.SPECIAL_CHAR_UNREGISTERED_TRADEMARK, iter_start, iter_insert, text_buffer)
                 elif iter_start.get_char() == cons.CHAR_STAR and iter_start.get_line_offset() == 0:
                     # "* " becoming "• " at line start
-                    dad.special_char_replace(cons.CHARS_LISTBUL[0], iter_start, iter_insert, text_buffer)
+                    dad.special_char_replace(config.CHARS_LISTBUL_DEFAULT[0], iter_start, iter_insert, text_buffer)
                 elif iter_start.get_char() == cons.CHAR_SQ_BR_CLOSE and iter_start.backward_char():
                     if iter_start.get_line_offset() == 0 and iter_start.get_char() == cons.CHAR_SQ_BR_OPEN:
                         # "[] " becoming "☐ " at line start
@@ -357,7 +364,7 @@ def on_sourceview_event_after_key_press(dad, text_view, event, syntax_highl):
                 elif iter_start.get_char() == cons.CHAR_COLON and iter_start.backward_char():
                     if iter_start.get_line_offset() == 0 and iter_start.get_char() == cons.CHAR_COLON:
                         # ":: " becoming "▪ " at line start
-                        dad.special_char_replace(cons.CHARS_LISTBUL[2], iter_start, iter_insert, text_buffer)
+                        dad.special_char_replace(config.CHARS_LISTBUL_DEFAULT[2], iter_start, iter_insert, text_buffer)
     return False
 
 def sourceview_cursor_and_tooltips_handler(dad, text_view, x, y):
@@ -550,6 +557,14 @@ def get_former_line_indentation(iter_start):
         return num_tabs*cons.CHAR_TAB
     return ""
 
+def get_pango_weight(is_bold):
+    """Get pango weight (integer 200:900) heavy=900, normal=400"""
+    return pango.WEIGHT_HEAVY if is_bold else pango.WEIGHT_NORMAL
+
+def get_pango_is_bold(weight):
+    """Get True if pango weight is bold (heavy)"""
+    return weight == pango.WEIGHT_HEAVY
+
 def windows_cmd_prepare_path(filepath):
     """Prepares a Path to be digested by windows command line"""
     return cons.CHAR_DQUOTE + filepath + cons.CHAR_DQUOTE
@@ -957,8 +972,8 @@ def dialog_image_handle(father_win, title, original_pixbuf):
     if response != gtk.RESPONSE_ACCEPT: return None
     return img_parms.original_pixbuf.scale_simple(int(img_parms.width), int(img_parms.height), gtk.gdk.INTERP_BILINEAR)
 
-def dialog_node_delete(father_win, warning_label):
-    """Confirmation before Node Remove"""
+def dialog_question_warning(father_win, warning_label):
+    """Question with Warning"""
     dialog = gtk.Dialog(title=_("Warning"),
         parent=father_win,
         flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
@@ -1460,100 +1475,97 @@ def dialog_selnode_selnodeandsub_alltree(dad, also_selection, also_include_node_
     if response != gtk.RESPONSE_ACCEPT: ret_val = 0
     return ret_val
 
-def set_bookmarks_menu_items(inst):
+def set_bookmarks_menu_items(dad):
     """Set Bookmarks Menu Items"""
-    bookmarks_menu = inst.ui.get_widget("/MenuBar/BookmarksMenu").get_submenu()
-    for menu_item in inst.bookmarks_menu_items:
+    bookmarks_menu = dad.ui.get_widget("/MenuBar/BookmarksMenu").get_submenu()
+    for menu_item in dad.bookmarks_menu_items:
         bookmarks_menu.remove(menu_item)
     menu_item = gtk.SeparatorMenuItem()
     menu_item.show()
     bookmarks_menu.append(menu_item)
-    inst.bookmarks_menu_items = [menu_item]
+    dad.bookmarks_menu_items = [menu_item]
     bookmarks_to_rm = []
-    for node_id_str in inst.bookmarks:
-        if not long(node_id_str) in inst.nodes_names_dict:
+    for node_id_str in dad.bookmarks:
+        if not long(node_id_str) in dad.nodes_names_dict:
             bookmarks_to_rm.append(node_id_str)
             continue
-        menu_item = gtk.ImageMenuItem(inst.nodes_names_dict[long(node_id_str)])
+        menu_item = gtk.ImageMenuItem(dad.nodes_names_dict[long(node_id_str)])
         menu_item.set_image(gtk.image_new_from_stock("pin", gtk.ICON_SIZE_MENU))
-        menu_item.connect("activate", select_bookmark_node, node_id_str, inst)
+        menu_item.connect("activate", select_bookmark_node, node_id_str, dad)
         menu_item.show()
         bookmarks_menu.append(menu_item)
-        inst.bookmarks_menu_items.append(menu_item)
-    for element in bookmarks_to_rm: inst.bookmarks.remove(element)
+        dad.bookmarks_menu_items.append(menu_item)
+    for element in bookmarks_to_rm: dad.bookmarks.remove(element)
 
-def set_menu_items_special_chars(inst):
+def set_menu_items_special_chars(dad):
     """Set Special Chars menu items"""
-    if not "special_menu_1" in dir(inst):
-        inst.special_menu_1 = gtk.Menu()
+    if not "special_menu_1" in dir(dad):
+        dad.special_menu_1 = gtk.Menu()
         first_run = True
     else:
-        children_1 = inst.special_menu_1.get_children()
+        children_1 = dad.special_menu_1.get_children()
         for children in children_1:
-            children.hide()
-            del children
+            children.destroy()
         first_run = False
-    for special_char in inst.special_chars:
+    for special_char in dad.special_chars:
         menu_item = gtk.MenuItem(special_char)
-        menu_item.connect("activate", insert_special_char, special_char, inst)
+        menu_item.connect("activate", insert_special_char, special_char, dad)
         menu_item.show()
-        inst.special_menu_1.append(menu_item)
+        dad.special_menu_1.append(menu_item)
     if first_run:
         # main menu
         special_menuitem = gtk.ImageMenuItem(_("Insert _Special Character"))
         special_menuitem.set_image(gtk.image_new_from_stock("insert", gtk.ICON_SIZE_MENU))
         special_menuitem.set_tooltip_text(_("Insert a Special Character"))
-        special_menuitem.set_submenu(inst.special_menu_1)
-        inst.ui.get_widget("/MenuBar/EditMenu").get_submenu().insert(special_menuitem, 14)
+        special_menuitem.set_submenu(dad.special_menu_1)
+        dad.ui.get_widget("/MenuBar/EditMenu").get_submenu().insert(special_menuitem, 14)
 
-def set_menu_items_recent_documents(inst):
+def set_menu_items_recent_documents(dad):
     """Set Recent Documents menu items on Menu and Toolbar"""
-    if not "recent_menu_1" in dir(inst):
-        inst.recent_menu_1 = gtk.Menu()
-        inst.recent_menu_2 = gtk.Menu()
+    if not "recent_menu_1" in dir(dad):
+        dad.recent_menu_1 = gtk.Menu()
+        dad.recent_menu_2 = gtk.Menu()
         first_run = True
     else:
-        children_1 = inst.recent_menu_1.get_children()
-        children_2 = inst.recent_menu_2.get_children()
+        children_1 = dad.recent_menu_1.get_children()
+        children_2 = dad.recent_menu_2.get_children()
         for children in children_1:
-            children.hide()
-            del children
+            children.destroy()
         for children in children_2:
-            children.hide()
-            del children
+            children.destroy()
         first_run = False
     for target in [1, 2]:
-        for i, filepath in enumerate(inst.recent_docs):
+        for i, filepath in enumerate(dad.recent_docs):
             if i >= cons.MAX_RECENT_DOCS: break
             menu_item = gtk.ImageMenuItem(filepath)
             menu_item.set_image(gtk.image_new_from_stock("gtk-open", gtk.ICON_SIZE_MENU))
-            menu_item.connect("activate", open_recent_document, filepath, inst)
+            menu_item.connect("activate", open_recent_document, filepath, dad)
             menu_item.show()
-            if target == 1: inst.recent_menu_1.append(menu_item)
-            else: inst.recent_menu_2.append(menu_item)
+            if target == 1: dad.recent_menu_1.append(menu_item)
+            else: dad.recent_menu_2.append(menu_item)
     if first_run:
         # main menu
         recent_menuitem = gtk.ImageMenuItem(_("_Recent Documents"))
         recent_menuitem.set_image(gtk.image_new_from_stock("gtk-open", gtk.ICON_SIZE_MENU))
         recent_menuitem.set_tooltip_text(_("Open a Recent CherryTree Document"))
-        recent_menuitem.set_submenu(inst.recent_menu_1)
-        inst.ui.get_widget("/MenuBar/FileMenu").get_submenu().insert(recent_menuitem, 3)
+        recent_menuitem.set_submenu(dad.recent_menu_1)
+        dad.ui.get_widget("/MenuBar/FileMenu").get_submenu().insert(recent_menuitem, 3)
         # toolbar
-        if inst.toolbar_open_n_recent >= 0:
+        if dad.toolbar_open_n_recent >= 0:
             menu_toolbutton = gtk.MenuToolButton("gtk-open")
             menu_toolbutton.set_tooltip_text(_("Open a CherryTree Document"))
             menu_toolbutton.set_arrow_tooltip_text(_("Open a Recent CherryTree Document"))
-            menu_toolbutton.set_menu(inst.recent_menu_2)
-            menu_toolbutton.connect("clicked", inst.file_open)
-            inst.ui.get_widget("/ToolBar").insert(menu_toolbutton, inst.toolbar_open_n_recent)
+            menu_toolbutton.set_menu(dad.recent_menu_2)
+            menu_toolbutton.connect("clicked", dad.file_open)
+            dad.ui.get_widget("/ToolBar").insert(menu_toolbutton, dad.toolbar_open_n_recent)
 
-def add_recent_document(inst, filepath):
+def add_recent_document(dad, filepath):
     """Add a Recent Document if Needed"""
-    if filepath in inst.recent_docs and inst.recent_docs[0] == filepath:
+    if filepath in dad.recent_docs and dad.recent_docs[0] == filepath:
         return
-    if filepath in inst.recent_docs: inst.recent_docs.remove(filepath)
-    inst.recent_docs.insert(0, filepath)
-    set_menu_items_recent_documents(inst)
+    if filepath in dad.recent_docs: dad.recent_docs.remove(filepath)
+    dad.recent_docs.insert(0, filepath)
+    set_menu_items_recent_documents(dad)
 
 def insert_special_char(menu_item, special_char, dad):
     """A Special character insert was Requested"""
@@ -1699,11 +1711,11 @@ def bookmarks_handle(dad):
     dad.ctdb_handler.pending_edit_db_bookmarks()
     return True
 
-def set_object_highlight(inst, obj_highl):
+def set_object_highlight(dad, obj_highl):
     """Set the Highlight to obj_highl only"""
-    if inst.highlighted_obj:
-        inst.highlighted_obj.drag_unhighlight()
-        inst.highlighted_obj = None
+    if dad.highlighted_obj:
+        dad.highlighted_obj.drag_unhighlight()
+        dad.highlighted_obj = None
     if obj_highl:
         obj_highl.drag_highlight()
-        inst.highlighted_obj = obj_highl
+        dad.highlighted_obj = obj_highl
